'
' ############ comments in german and english ##########################
' File: DDS_Control.bas
' Application: DDS AD9951 Ansteuerung mit ATmega8515
' Hardware: LCD + 4x3 Keypad + 4 Taster + Rotary Encoder + AD9951
' Compiler: BASCOM-AVR Version 1.8.11.1
' Programmer: Eigenbau (BASCOM auf STK200 gestellt)
' Used Flash: 48 % --> 3956 Byte
' Version: 0.9.11
' Last change: 30.04.2006
' (c) by Chris Hirt, Austria
' #######################################################################

' $Sim                                ' Simulation On/Off
$regfile = "m8515.dat"               ' ATmega8515
' $PROG &HFF,&H2F,&HD9,&H00           ' Fuse bits. Take care !
$crystal = 7372800
$hwstack = 64
$framesize = 64
$swstack = 64

' ----- Config LCD ----------
Config LCD = 16 * 2
Config LCDmode = PORT
Config LCDpin = Pin, RS = PortA.0, E = PortA.1
Config LCDpin = Pin, DB4 = PortA.2, DB5 = PortA.3, DB6 = PortA.4, DB7 = PortA.5
Config LCDbus = 4

'----- Config SPI ----------
DDRB = &b11111111
         ' Bit 0 = 1 unbenutzt; Output (PB.0, T0)
         ' Bit 1 = 1 unbenutzt; Output (PB.1, T1)
         ' Bit 2 = 1 RESET; Output     (PB.2, AIN0)
         ' Bit 3 = 1 IOUPT; Output     (PB.3, AIN1)
         ' Bit 4 = 1 CS; Output        (PB.4, SS)
         ' Bit 5 = 1 SDIO; Output      (PB.5, MOSI)
         ' Bit 6 = 1 unbenutzt; Output (PB.6, MISO)
         ' Bit 7 = 1 SCLK; Output      (PB.7, SCK)
SPCR = &b01010010  ' SPI Control Register
         ' Bit 0 = 0 SPR0 - 7372800Hz/64 = 115.2 kHz
         ' Bit 1 = 1 SPR1 - 7372800Hz/64 = 115.2 kHz
         ' Bit 2 = 0 CPHA - Clock Phase
         ' Bit 3 = 0 CPOL - Clock Polarity
         ' Bit 4 = 1 MSTR - Select as Master
         ' Bit 5 = 0 DORD - MSB first
         ' Bit 6 = 1 SPE  - SPI enabled
         ' Bit 7 = 0 SPIE - No SPI interupt if SPIF is set

' ----- Config other Ports -------
DDRC =  &b00001111          ' Rows - upper nibbles / Cols - lower nibble
PortC = &b11110000
DDRD.2 = 0                 ' PD.2 auf Eingang    (INT0)
PortD.2 = 0                ' und kein PullUp
DDRD.3 = 0                 ' PD.3 auf Eingang    (INT1)
PortD.3 = 1                ' und PullUp enabled
DDRD.4 = 0                 ' PD.4 auf Eingang
PortD.4 = 0                ' und kein PullUp

' ----- Config Timer --------------
Config TIMER1 = Timer, Prescale = 256
On TIMER1 isr_Timer1

' ----- Config Interrupts ---------
MCUCR.ISC00 = 1
MCUCR.ISC01 = 0                   ' any logical change
GIMSK.INT0 = 1
On INT0 isr_RotEncoder           ' Interrupt-Routine  Externer Int0
Config INT1 = Falling           ' falling edge --> interrupt request
On INT1 isr_Keys                 ' Interrupt-Routine  Externer Int1

' ----- Declarations --------------
Dim KeyCodeRow as Byte          ' Zeilen
Dim KeyCodeCol as Byte          ' Spalten
Dim KeyCode as Byte             ' Keycode
Dim KeyChar as String*1         ' gedrcktes Zeichen
Dim Freq as String*9            ' Frequenz als String
Dim Fno As Long                 ' Frequenz nummerisch
Dim dig as Byte                 ' Anzahl Ziffer
Dim dwn as Bit                  ' Hilfsvariable
Dim ton as Bit                  ' Hilfsvariable
Dim RotEnc as  Long             ' Drehgeber Impulszhler 1
Dim RotEncOld as Long           ' Drehgeber Impulszhler 2
Dim i as Byte                   ' Laufvariable
Dim L as Long                   ' Hilfsvariable
Dim CFR2(4) as Byte             ' Array with Control Register 2
Dim FTW(5) as Byte              ' Array with Frequency Tuning Word

Declare Sub ShowFreq       ' Frequenz anzeigen (F 000.000.000 Hz)
Declare Sub WriteAD9951    ' Write Frequency Tuning Word to AD9951

' ---------------------- Main -----------------------------------------------
Enable Interrupts              'Interrupts global zulassen
Enable INT0                    ' Externen Interrupt0 einschalten
Enable INT1                    ' Externen Interrupt1 einschalten
Enable TIMER1                  ' Timer 1 einschalten
Stop TIMER1                    ' Timer 1 stop
Timer1 = 62656
Cursor blink
Cls                             ' Clear LCD
KeyChar = ""
Freq = ""
dig = 0
Fno = 15000000                  ' Startwert der Frequenz
RotEnc = Fno
RotEncOld = RotEnc

'-------- AD9951 Init -------------
PortB = &b00010100    ' AD9951 Reset: RESET = 1, CS = 1
CFR2(1) = &b00000001  ' Steuerregister 2, Adresse 1
CFR2(2) = &b00000000  ' MSB
CFR2(3) = &b00000000  '
CFR2(4) = &b00100110  ' RefMult=4 / VCO Range=250-400 / Charge Pump=125uA
PortB = &b00000000    ' CS = 0, IOUPT = 0
For i = 1 To 4        ' Register schreiben
  SPDR = CFR2(i)      ' SPI Data Register
  While SPSR.7 = 0    ' SPI Status Register, Bit 7 = SPIF kontrollieren
  ' Warte bis SPIF = 1
  Wend
Next i
PortB = &b00001000     ' IOUPT = 1         ' strobe, Daten bernehmen
Waitus 50
PortB = &b00010000     ' IOUPT = 0, CS = 1

Do
  '------- Auswertung Rotary Encoder -----
  If RotEnc <> RotEncOld Then
    L = RotEnc - RotEncOld
    L = L * 100
    Fno = Fno + L
    If Fno < 10000 Then
      Fno = 10000
    End If
    RotEnc = Fno
    RotEncOld = RotEnc
    Call ShowFreq
    Call WriteAD9951
  End If

  '--------------------- Tastenauswertung ----------
  If KeyChar <> "" Then
    Select Case KeyChar
      Case "0" to "9": Freq = Freq + KeyChar   ' Ziffer setzen
                        Locate 1,dig
                        LCD KeyChar
      Case "#": If Freq <> "" Then     ' ENTER  = Zahl bernehmen
                    Fno = Val(Freq)
                    Call ShowFreq
                    Call WriteAD9951
                    dig = 0
                    Freq = ""
                    Locate 1,1
                    LCD SPC(16)
                    Cursor on
                  End  If
      Case "*": Locate 1, dig      ' DELETE = vorhergehende Zahl lschen
                 LCD SPC(1)
                 ShiftCursor Left
                 Decr dig
                 Freq = Left(Freq,dig)
      Case "U": dwn = 0                 ' UP
                 If ton = 1 Then
                   Start TIMER1
                 Else
                   Stop  TIMER1
                   Call ShowFreq
                   Call WriteAD9951
                 End If
                 Toggle ton
      Case "D": dwn = 1                 ' DOWN
                 If ton = 1 Then
                   Start TIMER1
                 Else
                   Stop  TIMER1
                   Call ShowFreq
                   Call WriteAD9951
                 End If
                 Toggle ton
      Case "L": Locate 2,1             ' Lock frequency
                 LCD "Lock"             ' derzeit nicht umgesetzt
      Case "S": Locate 2,1             ' Store frequency
                 LCD "Store"            ' derzeit nicht umgesetzt
    End Select
  KeyChar = ""
  End If
Loop

' ------------------------Subs ----------------------------------------------

Sub ShowFreq              ' Zeige Frequenz per LCD
  Local s  as String*9
  Local s1 as String*11
  Freq = Str(Fno)
  s = Format(Freq,"000000000")
  s1 = Left(s,3) + "." + Mid(s,4,3) + "." + Right(s,3)
  LowerLine
  Cursor Off
  LCD "F " ; s1 ; " Hz"
End Sub

Sub WriteAD9951                ' Schreibe Frequenz-Tuning-Word in AD9951
  Local ftwx as Long          ' FTW as Long
  Local fno2 as Long          ' Hilfsvariable
  Local i1 as Byte            ' Hilfsvariable
  Local lu1 as Byte           ' Hilfsvariable
  Local luM as Byte           ' Hilfsvariable
  Local luD as Long           ' Hilfsvariable
  Local y as Long             ' Hilfsvariable
  Local z as Long             ' Hilfsvariable
  Local HW as Word            ' Hilfsvariable
  Local LW as Word            ' Hilfsvariable
  ' FTW Berechnung im Long Format
  fno2 = Fno*2                     ' Doppelte Frequenz (!), anwendungsbezogen
  lu1 = LookUp(0,MULT)
  ftwx = lu1*L
  Restore DIVI
  For i = 0 To 7 Step 1
    i1 = i+1
    luM = LookUp(i1,MULT)
    y = luM*fno2
    Read luD               ' LookUp funktioniert hier nicht da luD = Long
    z = y/luD
    ftwx = ftwx + z
  Next i
  ' FTW nach AD9951 schreiben
  FTW(1) = &b00000100   ' FTW Adresse 4
  HW = HIGHW(ftwx)      ' Word 2
  LW  = ftwx            ' Word 1  (LW = LOWW(ftwx))
  FTW(2) = HIGH(HW)     ' Byte 4
  FTW(3) = LOW(HW)      ' Byte 3
  FTW(4) = HIGH(LW)     ' Byte 2
  FTW(5) = LOW(LW)      ' Byte 1
  PortB = &b00000000     ' CS = 0
  For i = 1 To 5        ' FTW in AD9951 schreiben
    SPDR = FTW(i)        ' SPI Data Register
    While SPSR.7 = 0    ' SPI Status Register, Bit 7 = SPIF kontrollieren
    ' Warte bis SPIF = 1
    Wend
  Next i
  PortB = &b00001000     ' IOUPT = 1         ' strobe, Daten bernehmen
  Waitus 50
  PortB = &b00010000     ' IOUPT =0, CS = 1
End Sub

' --------------------- Timer -----------------------------------------------
isr_Timer1:
  If dwn = 1 Then
    Fno = Fno - 1000
  Else
    Fno = Fno + 1000
  End If
  If Fno > 120000000 Then Fno = 120000000  ' maximaler Wert
  If Fno < 10000 Then Fno = 10000          ' mimimaler Wert
  Freq = Str(Fno)
  Locate 1,1
  LCD Freq
  Timer1 = 62656
Return

' --------------------- Externe Interrupts ---------------------------------

' ---- Read Encoder ----
isr_RotEncoder:                ' Externer Interrupt0 Routine
  Waitms 2
  If PinD.2 = 0 Then
    If PinD.4 = 0 Then Decr RotEnc Else Incr RotEnc
  End If
  GIFR.INTF0 = 1
Return

' ---- Tasten Auswertung ----
isr_Keys:                      ' Externer Interrupt1 Routine
  Waitms 100                  ' Entprellzeit in ms
  KeyCodeRow = PinC           ' Zeilen einlesen
  DDRC = &b11110000           ' change I/O nibbles
  PortC = &b00001111
  Waitms 1                    ' Port settle time
  KeyCodeCol = PinC            ' Spalten einlesen
  Select Case KeyCodeRow      ' KeyCode fr Zeilen zuordnen
    Case 112 : KeyCode = 12
    Case 176 : KeyCode = 8
    Case 208 : KeyCode = 4
    Case 224 : KeyCode = 0
    Case Else : KeyCode = 66
  End Select
  Select Case KeyCodeCol              ' KeyCode fr Spalten zuordnen
    Case 14 : KeyCode = KeyCode + 0
    Case 13 : KeyCode = KeyCode + 1
    Case 11 : KeyCode = KeyCode + 2
    Case  7 : KeyCode = KeyCode + 3
    Case Else : KeyCode = KeyCode + 66
  End Select
  If KeyCode > 15 Then KeyCode = 16
  KeyChar = LookupStr(KeyCode, DTA)   ' Zeichen aus LookUp Table
  Select Case KeyChar
    Case "0" To "9" : Incr dig
    If dig > 9 Then dig = 9            ' Maximal 9 Stellen
  End Select
  DDRC  = &b00001111                   ' Port return to origin state
  PortC = &b11110000
  GIFR.INTF1 = 1                       ' set INT1 bit
Return

' ----------------------- Data ---------------------------------------------
  DTA:
      Data "1", "2", "3", "D", "4", "5", "6", "U",
      Data "7", "8", "9", "S", "*", "0", "#", "L", "?"

  MULT:
      Data 10,7,3,7,4,1,8,2,4

  DIVI:
      Data 10&,100&,1000&,10000&,100000&,1000000&,10000000&,100000000&


' --------------------------- end of DDS-Control.bas --------------------